/*
 * Copyright 2008, Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

public class MenuForm
    extends Form
    implements CommandListener
{
    private final Command playCommand = new Command("Play new", Command.OK, 0);
    private final Command exitCommand = new Command("Exit", Command.EXIT, 0);
    private final Command continueCommand = new Command("Continue", Command.BACK, 1);

    private class OptDifficulty extends ChoiceGroup {
	OptDifficulty() {
	    super ("How Many Mines", Choice.EXCLUSIVE);
	    append("15", null);
	    setSelectedIndex(append("25", null), true);
	    append("35", null);
	    append("45", null);
	    append("55", null);
	    append("65", null);
	}

	public int howManyMines() {
	    return 10 * getSelectedIndex() + 15;
	}
    }

    private class OptBoard extends ChoiceGroup {
	OptBoard() {
	    super("Board Type", Choice.EXCLUSIVE);
	    append("6", null);
	    setSelectedIndex(append("8", null), true);
	}

	public MineField mineField(int w, int h,
				   boolean wrapX, boolean wrapY,
				   ParticipantSelector parSel) {
	    switch (getSelectedIndex()) {
	    case 0: return new MineField6(w, h, wrapX, wrapY, parSel);
	    case 1: return new MineField8(w, h, wrapX, wrapY, parSel);
	    default: return null;
	    }
	}
    }

    private class OptSelector8 extends ChoiceGroup {
	abstract class SelCtor {
	    public abstract String name();
	    public abstract ParticipantSelector build();
	    public boolean isDefault() { return false; }
	}
	private final SelCtor[] ctors = new SelCtor[] {
	    new SelCtor() {
		public String name() { return "4"; }
		public ParticipantSelector build() {
		    return new MineField8.ParSel4();
		}
	    },
	    new SelCtor() {
		public String name() { return "8"; }
		public ParticipantSelector build() {
		    return new ParticipantSelector();
		}
		public boolean isDefault() { return true; }
	    },
	    new SelCtor() {
		public String name() { return "8/4"; }
		public ParticipantSelector build() {
		    return new MineField8.ParSel84();
		}
	    },
	    new SelCtor() {
		public String name() { return "12"; }
		public ParticipantSelector build() {
		    return new MineField8.ParSel12();
		}
	    },
	    new SelCtor() {
		public String name() { return "16"; }
		public ParticipantSelector build() {
		    return new MineField8.ParSel16();
		}
	    }
	};

	OptSelector8() {
	    super("Neighborhood (board=8)", Choice.EXCLUSIVE);
	    for (int i = 0; i < ctors.length; ++i) {
		int idx = append(ctors[i].name(), null);
		if (ctors[i].isDefault())
		    setSelectedIndex(idx, true);
	    }
	}

	ParticipantSelector parSel() {
	    return ctors[getSelectedIndex()].build();
	}
    }

    private class OptConfiguration extends ChoiceGroup {
	private final boolean[] resultArray;
	private int numOptions = 0;

	private void addOption(String name, boolean on) {
	    setSelectedIndex(append(name, null), on);
	    ++numOptions;
	}

	OptConfiguration() {
	    super ("Configuration", Choice.MULTIPLE);
	    addOption("Deterministic", true);
	    addOption("Torus", false);
	    resultArray = new boolean[numOptions];
	}

	public boolean determinismEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[0];
	}

	public boolean torusEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[1];
	}
    }

    private final OptDifficulty optDifficulty = new OptDifficulty();
    private final OptBoard optBoard = new OptBoard();
    private final OptConfiguration optConfiguration = new OptConfiguration();
    private final OptSelector8 optSelector = new OptSelector8();

    private final MicroSweeper parent;

    public MenuForm(MicroSweeper parent) {
	super("MicroSweeper");
	this.parent = parent;

	append(optDifficulty);
	append(optBoard);
	append(optSelector);
	append(optConfiguration);

	addCommand(playCommand);
	addCommand(exitCommand);

	setCommandListener(this);
    }

    public int howManyMines() {
	return optDifficulty.howManyMines();
    }

    public boolean wrapX() {
	return optConfiguration.torusEnabled();
    }

    public boolean wrapY() {
	return wrapX();
    }

    public MineField mineField(int w, int h, boolean wrapX, boolean wrapY,
			       ParticipantSelector parSel) {
	return optBoard.mineField(w, h, wrapX, wrapY, parSel);
    }

    public ParticipantSelector parSel() {
	return optSelector.parSel();
    }

    public MineFieldAcceptor acceptor() {
	return optConfiguration.determinismEnabled()
	    ? new DeterministicMineFieldAcceptor() : null;
    }

    public void commandAction(Command c, Displayable s) {
	if (c == exitCommand)
	    parent.exitAction();
	else if (c == continueCommand)
	    parent.continueGame();
	else
	    parent.playAction();
    }

    private boolean gameActive = false;
    public void haveActiveGame(boolean status) {
	if (gameActive != status) {
	    if (status)
		addCommand(continueCommand);
	    else
		removeCommand(continueCommand);
	    gameActive = status;
	}
    }
}
